# 3rd Party Libraries and Contributions

- [Fluent Assertions](https://github.com/fluentassertions/fluentassertions) - A set of .NET extension methods that allow you to more naturally specify the expected outcome of a TDD or BDD-style test | [Apache License 2.0 (Apache)](https://github.com/dennisdoomen/fluentassertions/blob/develop/LICENSE)
- [xUnit.net](https://github.com/xunit/xunit) - Free, open source, community-focused unit testing tool for the .NET Framework | [Apache License 2.0 (Apache)](https://github.com/xunit/xunit/blob/main/license.txt)
- [Ian Griffith's `TimedLock`](http://www.interact-sw.co.uk/iangblog/2004/04/26/yetmoretimedlocking)
- [Steven van Deursen's `ReadOnlyDictionary`](https://blogs.cuttingedge.it/steven/posts/2007/readonlydictionary/) (until Polly v5.0.6)
- [Stephen Cleary's `AsyncEx` library](https://github.com/StephenCleary/AsyncEx) for `AsyncSemaphore` (supports `BulkheadAsync` policy for .NET 4.0 only) (until Polly v5.9.0) | [MIT license](https://github.com/StephenCleary/AsyncEx/blob/master/LICENSE)
- [@theraot](https://github.com/theraot)'s [`ExceptionDispatchInfo` implementation for .NET 4.0](https://stackoverflow.com/a/31226509/) (supports Timeout policy for .NET4.0 only) (until Polly v5.9.0) including also a contribution by [@migueldeicaza](https://github.com/migueldeicaza) | Licensed under and distributed under [Creative Commons Attribution Share Alike license](https://creativecommons.org/licenses/by-sa/3.0/) per [StackExchange Terms of Service](https://stackexchange.com/legal)
- Build powered by [Cake](https://cakebuild.net/) and [MinVer](https://github.com/adamralph/minver).  Developers powered by [Resharper](https://www.jetbrains.com/resharper/), with thanks to JetBrains for [OSS licensing](https://www.jetbrains.com/support/community/#section=open-source).

## Acknowledgements

- [lokad-shared-libraries](https://github.com/Lokad/lokad-shared-libraries) - Helper assemblies originally for .NET 3.5 and Silverlight 2.0 which were developed as part of the Open Source effort by Lokad.com (discontinued) | [New BSD License](https://raw.github.com/Lokad/lokad-shared-libraries/master/Lokad.Shared.License.txt)
- [@michael-wolfenden](https://github.com/michael-wolfenden) - The creator and mastermind of Polly!
- [@ghuntley](https://github.com/ghuntley) - Portable Class Library implementation.
- [@mauricedb](https://github.com/mauricedb) - Initial async implementation.
- [@robgibbens](https://github.com/RobGibbens) - Added existing async files to PCL project
- [Hacko](https://github.com/hacko-bede) - Added extra `NotOnCapturedContext` call to prevent potential deadlocks when blocking on asynchronous calls
- [@ThomasMentzel](https://github.com/ThomasMentzel) - Added ability to capture the results of executing a policy via `ExecuteAndCapture`
- [@yevhen](https://github.com/yevhen) - Added full control of whether to continue on captured synchronization context or not
- [@reisenberger](https://github.com/reisenberger) - Added full async cancellation support
- [@reisenberger](https://github.com/reisenberger) - Added async support for `ContextualPolicy`
- [@reisenberger](https://github.com/reisenberger) - Added `ContextualPolicy` support for circuit-breaker
- [@reisenberger](https://github.com/reisenberger) - Extended circuit-breaker for public monitoring and control
- [@reisenberger](https://github.com/reisenberger) - Added `ExecuteAndCapture` support with arbitrary context data
- [@kristianhald](https://github.com/kristianhald) and [@reisenberger](https://github.com/reisenberger) - Added `AdvancedCircuitBreaker`
- [@reisenberger](https://github.com/reisenberger) - Allowed async `onRetry` delegates to async retry policies
- [@Lumirris](https://github.com/Lumirris) - Add new `Polly.Net40Async` project/package supporting async for .NET 4.0 via Microsoft.Bcl.Async
- [@SteveCote](https://github.com/SteveCote) - Added overloads to `WaitAndRetry{Async}` methods that accept an `onRetry` delegate which includes the attempt count.
- [@reisenberger](https://github.com/reisenberger) - Allowed policies to handle returned results; added strongly-typed policies `Policy&lt;TResult&gt;`.
- [@christopherbahr](https://github.com/christopherbahr) - Added optimisation for circuit-breaker hot path.
- [@Finity](https://github.com/Finity) - Fixed circuit-breaker threshold bug.
- [@reisenberger](https://github.com/reisenberger) - Add some missing `ExecuteAndCapture{Async}` overloads.
- [@brunolauze](https://github.com/brunolauze) - Add CancellationToken support to synchronous executions (to support `TimeoutPolicy`).
- [@reisenberger](https://github.com/reisenberger) - Add `PolicyWrap`.
- [@reisenberger](https://github.com/reisenberger) - Add Fallback policy.
- [@reisenberger](https://github.com/reisenberger) - Add `PolicyKeys` and context to all policy executions, as bedrock for policy events and metrics tracking executions.
- [@reisenberger](https://github.com/reisenberger),  and contributions from [@brunolauze](https://github.com/brunolauze) -  Add Bulkhead Isolation policy.
- [@reisenberger](https://github.com/reisenberger) - Add Timeout policy.
- [@reisenberger](https://github.com/reisenberger) - Fix .NET Standard 1.0 targeting.  Remove `PCL259` target. `PCL259` support is provided via .NET Standard 1.0 target, going forward.
- [@reisenberger](https://github.com/reisenberger) - Fix CircuitBreaker `HalfOpen` state and cases when breakDuration is shorter than typical call timeout.  Thanks to [@vgouw](https://github.com/vgouw) and [@kharos](https://github.com/kharos) for the reports and insightful thinking.
- [@lakario](https://github.com/lakario) - Tidy CircuitBreaker `LastException` property.
- [@lakario](https://github.com/lakario) - Add `NoOpPolicy`.
- [@Julien-Mialon](https://github.com/Julien-Mialon) - Fixes, support and examples for .NET Standard compatibility with Xamarin PCL projects
- [@reisenberger](https://github.com/reisenberger) - Add mutable Context and extra overloads taking Context.  Allows different parts of a policy execution to exchange data via the mutable Context travelling with each execution.
- [@ankitbko](https://github.com/ankitbko) - Add `PolicyRegistry` for storing and retrieving policies.
- [@reisenberger](https://github.com/reisenberger) - Add interfaces by policy type and execution type.
- [@seanfarrow](https://github.com/SeanFarrow) - Add `IReadOnlyPolicyRegistry` interface.
- [@kesmy](https://github.com/Kesmy) - Migrate solution to msbuild15, banish `project.json` and `packages.config`
- [@hambudi](https://github.com/hambudi) - Ensure sync `TimeoutPolicy` with `TimeoutStrategy.Pessimistic` rethrows delegate exceptions without additional `AggregateException`.
- [@jiimaho](https://github.com/jiimaho) and [@Extremo75](https://github.com/ExtRemo75) - Provide public factory methods for `PolicyResult`, to support testing.
- [@Extremo75](https://github.com/ExtRemo75) - Allow fallback delegates to take handled fault as input parameter.
- [@reisenberger](https://github.com/reisenberger) and [@seanfarrow](https://github.com/SeanFarrow) - Add `CachePolicy`, with interfaces for pluggable cache providers and serializers.
- Thanks to the awesome devs at [@tretton37](https://github.com/tretton37) who delivered the following as part of a one-day in-company hackathon led by [@reisenberger](https://github.com/reisenberger), sponsored by [@tretton37](https://github.com/tretton37) and convened by [@thecodejunkie](https://github.com/thecodejunkie)
  - [@matst80](https://github.com/matst80) - Allow WaitAndRetry to take handled fault as an input to the `sleepDurationProvider`, allowing WaitAndRetry to take account of systems which specify a duration to wait as part of a fault response; e.g. Azure CosmosDB may specify this in `x-ms-retry-after-ms` headers or in a property to an exception thrown by the Azure CosmosDB SDK.
  - [@MartinSStewart](https://github.com/martinsstewart) - Add `GetPolicies()` extension methods to `IPolicyWrap`.
  - [@jbergens37](https://github.com/jbergens37) - Parallelize test running where possible, to improve overall build speed.
- [@reisenberger](https://github.com/reisenberger) - Add new `.HandleInner<TException>(...)` syntax for handling inner exceptions natively.
- [@rjongeneelen](https://github.com/rjongeneelen) and [@reisenberger](https://github.com/reisenberger) - Allow `PolicyWrap` configuration to configure policies via interfaces.
- [@reisenberger](https://github.com/reisenberger) - Performance improvements.
- [@awarrenlove](https://github.com/awarrenlove) - Add ability to calculate cache TTL based on item to cache.
- [@erickhouse](https://github.com/erickhouse) - Add a new `onBreak` overload that provides the prior state on a transition to an open state.
- [@benagain](https://github.com/benagain) - Bug fix: `RelativeTtl` in `CachePolicy` now always returns a ttl relative to time item is cached.
- [@urig](https://github.com/urig) - Allow `TimeoutPolicy` to be configured with `Timeout.InfiniteTimeSpan`.
- [@reisenberger](https://github.com/reisenberger) - Integration with [`IHttpClientFactory`](https://github.com/aspnet/HttpClientFactory/) for ASP.NET Core 2.1.
- [@freakazoid182](https://github.com/Freakazoid182) - `WaitAndRetry{Forever}` overloads where `onRetry` takes the retry number as a parameter.
- [@dustyhoppe](https://github.com/dustyhoppe) - Overloads where `onTimeout` takes thrown exception as a parameter.
- [@flin-zap](https://github.com/flin-zap) - Catch missing async continuation control.
- [@reisenberger](https://github.com/reisenberger) - Clarify separation of sync and async policies.
- [@reisenberger](https://github.com/reisenberger) - Enable extensibility by custom policies hosted external to Polly.
- [@seanfarrow](https://github.com/SeanFarrow) - Enable collection initialization syntax for `PolicyRegistry`.
- [@moerwald](https://github.com/moerwald) - Code clean-ups, usage of more concise C# members.
- [@cmeeren](https://github.com/cmeeren) - Enable cache policies to cache values of `default(TResult)`.
- [@aprooks](https://github.com/aprooks) - Build script tweaks for Mac and Mono.
- [@kesmy](https://github.com/Kesmy) - Add SourceLink support, clean up cake build.
- [@simluk](https://github.com/simluk) - Fix `continueOnCaptureContext` not being honored in async retry implementation (bug in v7.1.0 only).
- [@jnyrup](https://github.com/jnyrup) - Upgrade tests to Fluent Assertions v5.9.0
- [@SimonCropp](https://github.com/SimonCropp) - Add `netcoreapp3.0` target; code clean-ups.
- [@aerotog](https://github.com/aerotog) and [@reisenberger](https://github.com/reisenberger) - `IConcurrentPolicyRegistry` methods on `PolicyRegistry`
- [@reisenberger](https://github.com/reisenberger) and [@martincostello](https://github.com/martincostello) - Add RateLimit policy.
